/* VNA-lpg  dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This version is configured for the W8WWV board which has the
 * LTC2440 fast a/d chip installed.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 * I would like to thank Greg Ordy W8WWV for his help and ideas. It was 
 * his contribution of source code for windows ibraries known as VNAccess
 * and VNAgra that made this program possible. 
 * C++ code from VNAgra was converted to the dumb C compiler of the dspic.
 * This code is not as generalized as Gregs, and was
 * stripped to the bare minimum with intent of being being able to 
 * eventually run it on a dsPic30F4011 microcontroller.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */

// From the VNA Schematic:
//	LO DDS Data, DB25-3, LO DDS serial data in (D1)
//	RF DDS Data, DB25-2, RF DDS serial data in (D0)
//	DDS Reset, DB25-6, Shared DDS Reset in (D4)
//	DDS W_Clk, DB25-4, Shared DDS W_Clk in (D2)
//	DDS FQ_UD, DB25-5, Shared DDS FQ_UD in (D3)
//
//	ADC nCS_1, DB25-8, ADC nCS in (D6)
//	ADC SCK, DB25-7, ADC SCK in (D5)
//	ADC nSDO_1, DB25-11, inversion of ADC SDO out (IOBaseAddress+1, D7 [MSB])
//		Since this is the parallel port BUSY signal, which is inverted in
//		hardware at the computer, the BUSY read by software is the correct value!
//
//	VNA-Plus update: ADC (LTC2440) SDI, DB25-2, (D0)
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"


extern volatile unsigned short int timer_vna;
extern volatile int but3;		// enter button debounced in isr

//=============================================================================
// Stuff that may need to change for your hardware environment
//=============================================================================

//=============================================================================
// local defines.... you should not have to mess with these
//=============================================================================

//=============================================================================
// local global vars used by this program
//=============================================================================
// because of limited ram and eeprom for saving cal,
// we need to pull some tricks to get it all done
// structure with all calibration constants that is stored in eeprom
// aligned on 32 byte boundary
// seed eeprom with a few default values
// this forces a cal after a new pgm load
struct CAL _EEDATA(32) calEE={-1L,149999800L,0,0};

// structure in ram (filled in from eeprom at startup
struct CAL cal;

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void int_freq1( unsigned long freq );

// an alternate method is to save cal in code space...
// large calibration array in nonvolatile code space
// struct CAL cal[CALPTS] __attribute__ ((space(auto_psv))) ;
// reads are fine from C, but how to write into code space
// has not been figured out yet (needs asm utility)

//=============================================================================
// low level access to a short delay (about 1us)
// On the PC, the isa overhead makes bit flips take 1-2us
// with the dspic, the i/o is too screaming fast
// a loop setting i/o and repeatedly calling this routine
// gives a square wave with about 1.5us high and 1.5us low times.
// not exactly 1us, but close enough for retiming i/o.
//=============================================================================
void dly_1us( void )
{
	static long long int dummy;
	dummy++;
}
//=============================================================================
// low level access to AD9851 local osc data 
// PC - on parallel port pin 3, DO.1  
// dspic RB1
//=============================================================================
void AD9851_LODAT( int value )
{
    if ( value )
		_RB1 = 1;
    else
		_RB1 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 RF osc data 
// on parallel port pin 2, DO.0  
// dspic RB2
//=============================================================================
void AD9851_RFDAT( int value )
{
    if ( value )
        _RB2 = 1;
    else
        _RB2 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 reset line on both dds's 
// on parallel port pin 6, DO.4  
// dspic RB3
//=============================================================================
void AD9851_RESET( int value )
{
    if ( value )
        _RB3=1;
    else
        _RB3=0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 clk line on both dds's 
// on parallel port pin 4, DO.2  
// dspic RB6
//=============================================================================
void AD9851_CLK( int value )
{
    if ( value )
        _RB6 = 1;
    else
        _RB6 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 FQ_UD line on both dds's 
// on parallel port pin 5, DO.3
// dspic RB7  
//=============================================================================
void AD9851_FQUD( int value )
{
    if ( value )
        _RB7 = 1;
    else
        _RB7 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 chip serial clock bit 
// on parallel port pin 7, DO.5              (SDI data stable on rising edge)
// dspic RF0
//=============================================================================
void LTC2440_SCK( int value )
{
    if ( value )
        _RF0 = 1;
    else
        _RF0 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 chip select bit ( low gets 2440's attention )
// on parallel port pin 8, DO.6
// dspic RB8
//=============================================================================
void LTC2440_CS( int value )
{
    if ( value )
        _RB8 = 1;
    else
        _RB8 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 serial data out bit (used to read a/d value)
// on parallel port pin 11, +busy input on PC
// note, on the vna this bit is inverted in hardware and on pc... 
// dspic RF6
//=============================================================================
int LTC2440_SDO( void )
{
    if (_RF6)
        return(FALSE);
    else
        return(TRUE);
}

//=============================================================================
// low level access to LTC2440 serial data in bit(used to set conversion speed)
// on parallel port pin 2, D0.0
// This bit is also shared with the DDS data
// dspic 
//=============================================================================
void LTC2440_SDI( int value )
{
    if ( value )
        _RB2 = 1;
    else
		_RB2 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 A/D
// This routine tests to see ifthe A/D has finished a conversion and is ready
// to start another or have the results read out.
//=============================================================================
int LTC2440_Ready( void )
{
    int result;

	LTC2440_SCK(0);
    LTC2440_CS(0);
    LTC2440_CS(0);		// test hold cs low for 2us

    if (LTC2440_SDO() == FALSE)     // check for last conversion complete
        result = TRUE;
	else
		result = FALSE;

    LTC2440_CS(1);

    return result;
}

//=============================================================================
// low level access to LTC2440 A/D
// This routines reads the conversion results out of the A/D and returns
// status and voltage reading.
// - F0 is grounded, so the 2440 uses the internal 9mhz clock for conversion
//  see data sheet Figure 6. External Serial Clock, Single Cycle Operation
// Conversions may take up to 1/7 second depending on requested 
// conversion speed. See Table 3 on page 14 of ltc2440 docs for speed codes.
//=============================================================================
int LTC2440_Read( int speed_code, long int *voltage )
{
    int i;
	int rc = 0;

    if ( voltage )
		*voltage = 0;    

	// first sanity check ... does a/d become ready ?
	// longest conversion is 170ms
    for ( i=0; i<200; i++)
    {
        if (LTC2440_Ready())
            break;
		timer_vna = 10;			// 1ms delay
		while( timer_vna);
    }
    if ( i >= 200 )
    {
        printf("LTC2440 busy timeout(200ms). Is board connected and powered up?\r\n");
        return -1;
    }

	if ( cal.samplerate )
		LTC2440_SDI(1);			// do 7hz slow conversions
	else
		LTC2440_SDI(0);			// do 880hz fast conversions

    LTC2440_SCK(0);        	// make sure clock is low
    LTC2440_CS(0);         	// select a/d

	// start a conversion, and read data from last conversion
	for (i=0 ; i < 32 ; i++) 
    {
		if (voltage)
			*voltage = (*voltage << 1) + LTC2440_SDO();
		// pulse clock and read voltage data
        LTC2440_SCK(1);
	    LTC2440_SCK(0);    	// next data gets clocked out on falling edge
	}

	// second sanity check
	// d/a should have gone busy with a conversion started above
    if (LTC2440_Ready())
	{
		printf("LTC2440 not busy. Is board connected and powered up?\r\n");
		return -1;
	}

	if ( voltage )
	{
		// do some sanity checks on voltage
		// top 2 bits should be 0 (/eoc and a 0)
		if (( *voltage & 0xc0000000L ) != 0)
		{
        	printf("   ***LTC2440 invalid data format 0x%08lX\r\n",*voltage);
        	rc= -1;
		}
		if (( *voltage & 0x30000000L ) == 0x30000000L)
		{
        	printf("   ***LTC2440 +ve overrange\r\n");
        	rc= -1;
		}
		if (( *voltage & 0x30000000L ) == 0)
		{
        	printf("   ***LTC2440 -ve underrange\r\n");
        	rc= -1;
		}
		if ( rc == 0)
		{
			// if its a good reading, narrow it down to useful 24 bit reading
			// set sign bit
			*voltage = *voltage << 3;
			// discard low trash bits and propagate sign bit down
			*voltage = *voltage >> 8;
		}
	}
    return rc;
}

//=============================================================================
// function to convert frequency to dds tuning word
// done in 64 bit math with rounding
// for best accuracy, refclk and f0 can be passed in in 0.1hz units
//=============================================================================
unsigned long int VNA_FreqToTW(unsigned long int RefClk, unsigned long int F0)
{
	long long int x;
	x = F0;
	x <<= 32;
	x += (RefClk / 2);
	x /= RefClk;
	return (unsigned long int) x;
}


//=============================================================================
// Routine to set the rf DDS and the LO DDS to the specified freq and the LO 
// to the specified phase offset.
//=============================================================================
void AD9851_SetFreq( unsigned long int hz, unsigned long int phase )
{
	int i;
	unsigned long int divisor;
	divisor =  VNA_FreqToTW(cal.refclk,hz);
//	printf("  %u Hz tuning word = %u\n",hz,divisor);

	// set  freq(32) from divisor
	for (i=0; i<32; i++)
	{
		if (divisor & 0x00000001L)
		{
			AD9851_RFDAT(1);
			AD9851_LODAT(1);
		}
		else
		{
			AD9851_RFDAT(0);
			AD9851_LODAT(0);
		}
		AD9851_CLK(1);
		AD9851_CLK(0);
		divisor >>= 1;
	}

	// set direct osc(1), test bit(1),powerup both DDS chips (1)
	AD9851_LODAT(0);
	AD9851_RFDAT(0);
	for (i=0; i<3; i++)
	{
		AD9851_CLK(1);
		AD9851_CLK(0);
	}

	// set zero or 90 deg phase angle (5)
	AD9851_RFDAT(0);
	for (i=0; i<5; i++)
	{
		if ( (phase == 90) && (i == 3) )
			AD9851_LODAT(1);
		else
			AD9851_LODAT(0);
		AD9851_CLK(1);
		AD9851_CLK(0);
	}
	// now activate the new configuration
	AD9851_FQUD(1);
	AD9851_FQUD(0);
}
//=============================================================================
// Routine to reset the rf DDS and the LO DDS.
// This resets the DDS chips and preps them for receiving 40bit serial 
// command words.
// DO NOT MESS WITH THIS ROUTINE!!!!!!!
// it is very hard to get the DDS's to power up and be synced in a consistant
// configuration. This is critical if saved calibrations are to be used.
// Getting this right took as much time as writting all the other code in
// this project!
//=============================================================================
void AD9851_Reset( void )
{
	int i;
	// set dds lines in a known position
	AD9851_CLK(0);
	AD9851_FQUD(0);
	AD9851_RESET(0);
	AD9851_LODAT(0);
	AD9851_RFDAT(0);

	// reset both dds ( needs to be > 5 ref clk periods )
	AD9851_RESET(1);
	AD9851_RESET(0);

	// enter DDS serial programming mode
	AD9851_CLK(1);
	AD9851_CLK(0);
	AD9851_FQUD(1);
	AD9851_FQUD(0);
	// we are now in serial mode but not synced

	// clock a power down word into 40 bit SR
	for( i=0; i<34; i++)
	{
		AD9851_CLK(1);		// clocks 0..33
		AD9851_CLK(0);
	}
	AD9851_LODAT(1);		// set power down bit
	AD9851_RFDAT(1);
	AD9851_CLK(1);			// clock 34
	AD9851_CLK(0);
	AD9851_LODAT(0);
	AD9851_RFDAT(0);

	for( i=0; i<5; i++)
	{
		AD9851_CLK(1);		// clocks 35..39
		AD9851_CLK(0);
	}

	// reset both dds ( needs to be > 5 ref clk periods )
	AD9851_RESET(1);
	AD9851_RESET(0);

	// clock power down word into core
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	// enter DDS serial programming mode
	AD9851_CLK(1);
	AD9851_CLK(0);
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	// we are now in serial mode, synced and powered down
	// now clock out 40 zeros to power us up
	for( i=0; i<40; i++)
	{
		AD9851_CLK(1);		// clocks 0..39
		AD9851_CLK(0);
	}
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	AD9851_SetFreq( 10000000, 90 );
}

//=============================================================================
// setup port access and init vars for VNA
//=============================================================================
int VNA_Open(void)
{
    LTC2440_CS(1);          // unselect a/d
    LTC2440_SCK(0);         // clock low

	AD9851_Reset();			// get DDS into serial command mode

	LTC2440_Read(0x0f,NULL);		// dummy read to init a/d
    return (0); 			// all ok
}

//=============================================================================
// puts vna to sleep ( low power consumption )
// power down DDS chips, turn off status LED turn off encoder LED.
// --------this routine is unused and untested-------
// srategy - call repeatedly, watch a variable incremented by timer
// timer resets variable on keypressed, or serial transaction
// if vraiable exceeds some threshold, power everything possible down and put
// a indication on LCD once a sleep we stay in this routine until a key is pressed
// or a serial byte is received. On wake up, we power up devices and return.
// The tricky partis making sure everyone who calls us will work properly after a power
// down event.
//=============================================================================
int VNA_Idle(void)
{
	int i;
	// clock a power down word into 40 bit SR
	for( i=0; i<34; i++)
	{
		AD9851_CLK(1);		// clocks 0..33
		AD9851_CLK(0);
	}
	AD9851_LODAT(1);		// set power down bit
	AD9851_RFDAT(1);
	AD9851_CLK(1);			// clock 34
	AD9851_CLK(0);
	AD9851_LODAT(0);
	AD9851_RFDAT(0);

	for( i=0; i<5; i++)
	{
		AD9851_CLK(1);		// clocks 35..39
		AD9851_CLK(0);
	}

	// clock power down word into core
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	STATUS_LED = 1;		// led off
	ENC_LED = 1;		// turn off encoder led

    return (0); 			// all ok
}

//=============================================================================
// function to convert d/a counts to microvolts 
// done in 64 bit math with rounding
//=============================================================================
long int VNA_CntsToVolts(long int cnts)
{
	signed long long int x;
	// uvolts = (cnts * -2500000L) /(2 ^ 24);
	x = (long long)cnts * -2500000LL;
	x >>= 24;
	return (long int) x;
}


//=============================================================================
//
// Compute One-Port Calibration method error terms from three independent 
// measurements of calibration standards. Open, Short and Load
//
// These three error terms are then used by 
// VNA_ComputeActualReflectionCoefficient() to turn the measurement
// of an unknown into an actual reflection coefficient.
//
// M1 is the measured reflection coefficient for the first calibration standard.
// A1 is the actual reflection coefficient for the first calibration standard.
// M2 is the measured reflection coefficient for the second calibration standard.
// A2 is the actual reflection coefficient for the second calibration standard.
// M3 is the measured reflection coefficient for the third calibration standard.
// A3 is the actual reflection coefficient for the third calibration standard.
//
//=============================================================================
int VNA_ComputeOnePortCalibrationErrorTerms
                        ( __complex__ double M1,
						  __complex__ double A1,
					      __complex__ double M2,
						  __complex__ double A2,
					      __complex__ double M3,
						  __complex__ double A3,
					      __complex__ double *pe00,
                          __complex__ double *pe11,
                          __complex__ double *pDeltae)

{
	__complex__ double e00, e11,Deltae;

	// Check for independence...
	if ((A1==A2) || (A1==A3) || (A2==A3))
		return FALSE;
	if ((M1==M2) || (M1==M3) || (M2==M3))
		return FALSE;

	//
	// This is an analytic solution (via substitution) of the following 3 linear equations in 3 unknowns:
	//	e00 + A1*M1*e11 - A1*Deltae = M1
	//	e00 + A2*M2*e11 - A2*Deltae = M2
	//	e00 + A3*M3*e11 - A3*Deltae = M3
	//

	// Compute Deltae
	Deltae = (((A2*M2 - A1*M1)*(M1-M3)) + ((A3*M3 - A1*M1)*(M2-M1))) /
              (((A2*M2 - A1*M1)*(A3-A1)) - ((A3*M3 - A1*M1)*(A2-A1)));

	// Compute e11
	e11 = (M2 - M1 + (A2-A1)*Deltae) / (A2*M2 - A1*M1);

	// Compute e00
	e00 = M1 - A1*M1*e11 + A1*Deltae;

	// Distribute the results
	if (pe00)
		*pe00 = e00;
	if (pe11)
		*pe11 = e11;
	if (pDeltae)
		*pDeltae = Deltae;

//    printf("VNA_ComputeOnePortCalibrationErrorTerms()...\ne00 = %f %fi, e11 = %f %fi, Deltae=%f %fi\n",
//        creal(e00),cimag(e00),creal(e11),cimag(e11),creal(Deltae),cimag(Deltae));
            
	return TRUE;
}
//=============================================================================
// Computes an actual reflection coefficient from a measured reflection 
// coefficient and error terms.
//=============================================================================
int VNA_ComputeReflCoeff (
		__complex__ double RhoM, 
		__complex__ double e00, 
		__complex__ double e11, 
		__complex__ double Deltae,
		__complex__ double *pRhoA)
{
	__complex__ double RhoA;

	RhoA = (RhoM - e00) / (RhoM*e11 - Deltae);

	if (pRhoA)
		*pRhoA = RhoA;
//    printf("VNA_ComputeActualReflectionCoefficient()... RhoA = %f %fi\n",creal(RhoA),cimag(RhoA));
	return TRUE;
}

//=============================================================================
// Computes a complex impedance (Zx) from an actual reflection coefficient (RhoA)
// and a base impedance (Z0 - usually 50+0i).
//=============================================================================
int VNA_ComputeComplexImpedance(
		__complex__ double RhoA, 
		__complex__ double Z0,
		__complex__ double *pZx )
{
	__complex__ double  One = 1+0i;
	__complex__ double 	Zx;

	//
	// This is an analytic solution (by rearranging terms) of:
	//
	//	RhoA = (Zx - Z0) / (Zx + Z0);
	//

	// Check for division by 0, which implies infinite impedance.
	if (RhoA == One) 
	{
		if (pZx)
			*pZx = 1.7E308+1.7E308i;
		return FALSE;
	}

	// Compute Zx
	Zx = Z0 * (One + RhoA) / (One - RhoA);

	if (pZx)
		*pZx = Zx;

	return TRUE;
}

//=============================================================================
// Computes capacitance from a complex impedance and frequency
//=============================================================================
int VNA_ComputeCapacitance (double ImZx, double Freq, double *pCap)
{
	*pCap = 0.0;
	if (ImZx >= 0.0)
		return FALSE;

	*pCap = -1.0 / (2.0 * M_PI * Freq * ImZx);
	return TRUE;
}

//=============================================================================
// Computes inductance from a complex impedance and frequency
//=============================================================================
int VNA_ComputeInductance(double ImZx, double Freq, double *pInd)
{
	*pInd = 0.0;

	if (Freq == 0.0 || ImZx < 0.0)
		return FALSE;

	*pInd = ImZx / (2.0 * M_PI * Freq);

	return TRUE;
}

//=============================================================================
// Computes Q from a complex impedance.
//=============================================================================
int VNA_ComputeQ(__complex__ double Zx, double *pQ)
{
	if ( pQ == NULL )
		return FALSE;

	if (__real__ Zx <= 0.0) 
	{
		*pQ = 0.01;
		return FALSE;
	}

	*pQ = __imag__ Zx / __real__ Zx;
	if (*pQ < 0.0)
	 	 *pQ = -*pQ;

	return TRUE;
}

//=============================================================================
// Compute series from parallel.
//=============================================================================
int VNA_ComputeSeriesFromParallel(double Rp, double Xp, double *pRs, double *pXs)
{
	if ((pRs == NULL) || (pXs == NULL))
		return FALSE;
	else
	{
		*pRs = (Rp * Xp * Xp) / ((Rp * Rp) + (Xp * Xp));
		*pXs = (Rp * Rp * Xp) / ((Rp * Rp) + (Xp * Xp));
		return TRUE;
	}
}
//=============================================================================
// Compute parallel from series.
//=============================================================================
int VNA_ComputeParallelFromSeries(double Rs, double Xs, double *pRp, double *pXp)
{
	if ((pRp == NULL) || (pXp == NULL))
		return FALSE;
	else
	{
		*pRp = (Rs * Rs + Xs * Xs) / Rs;
		*pXp = (Rs * Rs + Xs * Xs) / Xs;
		return TRUE;
	}
}
//=============================================================================
// Computes magnitude of a complex number
//=============================================================================
double my_cabs( __complex__ double x )
{
	return sqrt( __real__ x * __real__ x) + (__imag__ x * __imag__ x );
}

//=============================================================================
// Computes SWR given a reflection coefficient.
//=============================================================================
int VNA_ComputeSWR(__complex__ double Rho, double *pSWR)
{
	double RhoMag = my_cabs( Rho );
	if (pSWR == NULL)
		return FALSE;
	else
	{
		*pSWR = (1.0 + RhoMag) / (1.0 - RhoMag);
		return TRUE;
	}
}

//=============================================================================
// Computes Return Loss [RL]) given a reflection coefficient.
//=============================================================================
int VNA_ComputeRL(__complex__ double Rho, double *pRL)
{
	double RhoMag = my_cabs( Rho );
	if (pRL == NULL)
		return FALSE;
	else
	{
		*pRL = -20.0 * log10 (RhoMag);
		return TRUE;
	}
}

//=============================================================================
// Polar-to-Rectangular conversion.
// Angles are expressed in Radians.
//=============================================================================
int VNA_PToR (double Mag, double Angle, double *pRe, double *pIm)
{
	if ((pRe==NULL || pIm==NULL))
		return FALSE;
	else
	{
		*pRe = Mag * cos(Angle);
		*pIm = Mag * sin(Angle);
		return TRUE;
	}
}

//=============================================================================
// Rectangular-to-Polar conversion.
// Angles are expressed in Radians.
//=============================================================================
int VNA_RToP (double Re, double Im, double *pMag, double *pAngle)
{
	if ( (pMag == NULL) || (pAngle == NULL))
		return FALSE;
	else
	{
		*pMag = sqrt( (Re*Re)+(Im*Im) );
		*pAngle = atan2(Im,Re);
		return TRUE;
	}
}

//=============================================================================
// Degrees-to-Radians conversion.
//=============================================================================
int VNA_DToR(double Degrees, double *pRadians)
{
	if (pRadians)
	{
		*pRadians = 2.0 * M_PI * Degrees / 360.0;
		return TRUE;
	}
	else
		return FALSE;
}

//=============================================================================
// Radians-to-Degrees conversion.
//=============================================================================
int VNA_RToD (double Radians, double *pDegrees)
{
	if (pDegrees)
	{
		*pDegrees = 360.0 * Radians / (2.0 * M_PI);
		return TRUE;
	}
	else
		return FALSE;
}
#if 0  // this code is broken
//=============================================================================
//
// Adjusts an "ideal" reflection coefficient.
//	RhoIdeal		The ideal reflection coefficient
//	R				Series resistance (in Ohms)
//	L				Series inductance (in nanohenrys)
//	C				Shunt capacitance (in picofarads)
//	Freq			Frequency (in Hz)
//	Z0				base impedance (usually (50, 0))
//
//=============================================================================
int VNA_AdjustIdealReflectionCoefficient (
		__complex__ double RhoIdeal,
		double R, double L, double C,
		double Freq,
		__complex__ double Z0,
		__complex__ double *pRho)
{
	__complex__ double Zideal;
	__complex__ double Zadjusted;
	__complex__ double RhoAdjusted;
	__complex__ double One = 1.0+0.0I;
	__complex__ double Zero = 0.0+0.0I;

	double Cx = 0.0;	// Capacitive Reactance
	double Lx = 0.0;	// Inductive Reactance

	// start with the ideal value of Rho
	if (pRho)
		*pRho = RhoIdeal;

	// Compute capacitive reactance:
	if (C > 0.0)
		Cx = 1.0 / (2.0 * M_PI * Freq * (C * 1.0E-12));

	// Compute inductive reactance:
	if (L > 0.0)
		Lx = 2.0 * M_PI * Freq * (L * 1.0E-9);

	// If no adjustments are specified, we are done.
	if (R == 0.0 && Cx == 0.0 && Lx == 0.0)
		return TRUE;

	// The ideal reflection  coefficient of an open circuit, is (1, 0).
	// This is a special case, since it can't be converted to a complex impedance.
	if ( creal(RhoIdeal) == 1.0 && cimag(RhoIdeal) == 0.0) 
	{
		if (R == 0.0 && Lx == 0.0) 
		{
			// Zadjusted is the shunt capacitance.
			__real__ Zadjusted = 0.0;
			__imag__ Zadjusted = -Cx;
			goto ComputeAdjustedRho;
		}
	}
	else
		Zideal = Z0 * (One + RhoIdeal) / (One - RhoIdeal);

	// Adjust the ideal complex impedance
	// First the easy DC resistance and inductive reactance
	__real__ Zadjusted = creal(Zideal) + R;
	__imag__ Zadjusted = cimag(Zideal) + Lx;

	// Now the shunt capacitance
	if (Cx > 0.0) 
	{
		if ( Zadjusted == Zero )
		{	
			__real__ Zadjusted = 0.0;
			__imag__ Zadjusted = -Cx;
		}
		else
			Zadjusted = One / ((One / Zadjusted) + (One / ( 0.0 - CxI)));
	}

ComputeAdjustedRho:
	RhoAdjusted = (Zadjusted - Z0) / (Zadjusted + Z0);
	if (pRho)
		*pRho = RhoAdjusted;
	return TRUE;
}
#endif  // of broken code

//=============================================================================
// gets a raw bridge reading and returns a complex number (I and Q)
// which is used during cal or measurements to compute a relection coeff.
//=============================================================================
int VNA_GetReading( long int freq, __complex__ double *pResult )
{
	long int volts, volts90;

	AD9851_SetFreq( freq, 0 );
	timer_vna = 100;		// 10ms delay
	while( timer_vna );

	LTC2440_Read(0x0f,NULL);	// start conversion
	LTC2440_Read(0x0f,&volts);	// get results
	AD9851_SetFreq( freq, 90 );
	timer_vna = 100;		// 10ms delay
	while( timer_vna );

	LTC2440_Read(0x0f,NULL);
	LTC2440_Read(0x0f,&volts90);
//	printf("%8ld %8ld %8ld\r\n",freq,VNA_CntsToVolts(volts),
//              VNA_CntsToVolts(volts90));
	if (pResult)
	{
		__real__(*pResult) = VNA_CntsToVolts(volts)/1E6;
		__imag__(*pResult) = VNA_CntsToVolts(volts90)/1E6;

//	    printf("Result =%f %+fi\r\n",creal(*pResult), cimag(*pResult) );
		return TRUE;
	}
	else
		return FALSE;
}

//=============================================================================
// code to interpolate the calibration constants
// feed it a freq and it will fill in the 3 error terms
// by using linear interpolation from the stored cal constants
// this code is used in run mode and in remote mode
// returns a negative number if freq is out of cal table range
//=============================================================================
int VNA_interpcal( long int freq,
					__complex__ double *e00,
					__complex__ double *e11,
					__complex__ double *Deltae )
{
	double cif;
	long int fmax;
    int ci;          // index into calibration structure
	int rc = 0;

	fmax = cal.f_start + ((cal.f_num -1)*cal.f_step);

	if ( freq < cal.f_start )
	{
		printf("freq too low %ld\r\n",freq);
		freq = cal.f_start;
		rc = -1;
	}
	if ( freq > fmax ) 
	{
		printf("freq too high %ld\r\n",freq);
		freq = fmax;
		rc = -2;
	}

	cif = (double)(freq - cal.f_start) / (double)cal.f_step;
	ci = (int)cif;
	cif = cif - (double)ci;
//	printf("freq=%ld cif=%f ci=%d\r\n",freq,cif,ci);

	if ( ci < (cal.f_num-1) )
	{
		// interpolate cal pts between index [ci] and [ci+1]
		*e00 = cal.cal_out.e00[ci+1] * cif +
 					cal.cal_out.e00[ci] * (1.0-cif);
//	printf("e00[ci]=%f %f e00[ci+1]=%f %f\r\n",
//	__real__ cal.cal_out.e00[ci], __imag__ cal.cal_out.e00[ci],
//	__real__ cal.cal_out.e00[ci+1], __imag__ cal.cal_out.e00[ci+1]);
//	printf("e00[x]=%f %f\r\n",__real__ *e00, __imag__ *e00);

		*e11 = cal.cal_out.e11[ci+1] * cif +
 				cal.cal_out.e11[ci] * (1.0-cif);
		*Deltae = cal.cal_out.Deltae[ci+1] * cif +
 				cal.cal_out.Deltae[ci] * (1.0-cif);
	}
	else
	{
		// we are right at the top of the range... use top cal pts directly
		*e00 = cal.cal_out.e00[cal.f_num-1];
		*e11 = cal.cal_out.e11[cal.f_num-1];
		*Deltae = cal.cal_out.Deltae[cal.f_num-1];
	}
	return rc;
}


//=============================================================================
// code to generate a calibration file
//=============================================================================
void VNA_calibrate( void )
{
	long int freq;
    long int ci;

	// theoretical OSL standards
	__complex__ double A1 =  1.0+0.0I;       // open
	__complex__ double A2 = -1.0+0.0I;       // short
	__complex__ double A3 =  0.0+0.0I;       // 50ohm load
	__complex__ double M1;					 // temp for measured short
	__complex__ double M2;					 // temp for measured open
	__complex__ double M3;					 // temp for measured 50ohm

	// bounds check calibration params
	if (( cal.f_num < 2 || cal.f_num > 40 ) ||	// validate number of cal points
		( cal.f_start < MINFREQ ) ||			// validate minimum freq
		( cal.f_start > MAXFREQ) ||
		( cal.f_step < 10 ) ||					// validate step size
		( cal.f_step > CALSTEP ))
	{
		// setup defaults
		cal.f_num = CALPTS;
		cal.f_start = CALSTART;
		cal.f_step = CALSTEP;
		lcd_goto(0x40);
		lcd_puts("Using Defaults...      ");
		while(but3 < 0);	// wait for enter button dwn
		while(but3 > 0);	// wait for enter button up
	}

    // do a calibration run and collect data from the 3 OSL standards
	lcd_goto(0x40);
	lcd_puts("Attach open standard    ");
	while(but3 < 0);	// wait for enter button dwn
	while(but3 > 0);	// wait for enter button up
	lcd_goto(0x40);
	lcd_puts("      ");

    for (ci = 0; ci < cal.f_num; ci++)
    {
        freq = ci * cal.f_step + cal.f_start;
	    AD9851_SetFreq( freq, 0 );
		timer_vna = 100;		// 10ms delay
		lcd_goto(0);
		lcd_puts("CAL                     ");
		int_freq1(freq);
		while(timer_vna);
	    VNA_GetReading( freq, &cal.cal_in.M1[ci] );
    }

	lcd_goto(0x40);
	lcd_puts("Attach short standard   ");
	while(but3 < 0);	// wait for enter button dwn
	while(but3 > 0);	// wait for enter button up
	lcd_goto(0x40);
	lcd_puts("      ");

    for (ci = 0; ci < cal.f_num; ci++)
    {
        freq = ci * cal.f_step + cal.f_start;
	    AD9851_SetFreq( freq, 0 );

		timer_vna = 100;		// 10ms delay
		lcd_goto(0);
		lcd_puts("CAL                     ");
		int_freq1(freq);
		while(timer_vna);

	    VNA_GetReading( freq, &cal.cal_in.M2[ci] );
    }

	lcd_goto(0x40);
	lcd_puts("Attach 50 ohm standard  ");
	while(but3 < 0);	// wait for enter button dwn
	while(but3 > 0);	// wait for enter button up
	lcd_goto(0x40);
	lcd_puts("      ");

    for (ci = 0; ci < cal.f_num; ci++)
    {
        freq = ci * cal.f_step + cal.f_start;
	    AD9851_SetFreq( freq, 0 );

		timer_vna = 100;		// 10ms delay
		lcd_goto(0);
		lcd_puts("CAL                     ");
		int_freq1(freq);
		while(timer_vna);
	    VNA_GetReading( freq, &cal.cal_in.M3[ci] );
    }

	lcd_goto(0x40);
	lcd_puts("Computing Calibration...");
    for (ci = 0; ci < cal.f_num; ci++)
    {
		// load temps because cal routine will over write the raw readins
		M1 = cal.cal_in.M1[ci];
		M2 = cal.cal_in.M2[ci];
		M3 = cal.cal_in.M3[ci];

	    VNA_ComputeOnePortCalibrationErrorTerms(M1,A1,
                                                M2,A2,
                                                M3,A3,
					     					    &cal.cal_out.e00[ci], 
                                                &cal.cal_out.e11[ci], 
                                                &cal.cal_out.Deltae[ci]);
    }
}



#if 0
//=============================================================================
// test code for checking VNA
//=============================================================================
int main( int argc, char *argv[] )
{

//    FILE *fp;
    int rc;
	int freq;


	// unknown reading
	__complex__ double unknown;		// raw voltages
	__complex__ double Rho;			// computed reflection coeff.
	__complex__ double Zx;			// computed impedance
	__complex__ double Z50 = 50.0+0.0I;

	double SWR;					// computed SWR

    int ci;                     // index into calibration structure

	// Open the VNA.
	rc = VNA_Open();

    if ( VNA_LoadCalibration("cal.txt") == FALSE )
        // do a OSL calibration and save results to cal.txt
        VNA_calibrate();

#if 0
    
	printf("attach unknown\n");
	getchar();	
    fp = fopen("unknown.loads","wt");       // linsmith input file
    fprintf(fp,"<?xml version =\"1.0\"?>\n");
    fprintf(fp,"<loads>\n");
    
    for (ci = 0; ci < CALPTS; ci++)
    {
        freq = ci * CALSTEP + CALSTART;
	    AD9851_SetFreq( freq, 0 );
	    usleep(10000);
	    VNA_GetReading( freq, &unknown );
	    VNA_ComputeActualReflectionCoefficient( unknown,cal[ci].e00,cal[ci].e11,cal[ci].Deltae,&Rho);
	    VNA_ComputeComplexImpedance(Rho, Z50, &Zx);
	    VNA_ComputeSWR(Rho, &SWR);

//	    printf("Frequency(MHz) = %f  ",(double)freq/1E6);
//	    printf("Impedance =%f %fi  ",creal(Zx),cimag(Zx));
//	    printf("Swr       =%f \n",SWR);
//      fprintf(fp," <load f=\"%f\" r=\"%f\" x=\"%f\" />\n", 
//               (double)freq/1e6, creal(Zx), cimag(Zx));
               
    }
    fprintf(fp,"</loads>\n");
    fclose(fp);
    
#endif
    
#if 0
        freq = 1000000L;
	    AD9851_SetFreq( freq, 0 );
	    usleep(100000);

	    // do a calibration at this freq
	    printf("attach open standard\n");
	    getchar();	
	    VNA_GetReading( freq, &M1 );
	    printf("attach short standard\n");
	    getchar();
	    VNA_GetReading( freq, &M2 );
	    printf("attach 50 ohm standard\n");
	    getchar();
	    VNA_GetReading( freq, &M3 );

	    VNA_ComputeOnePortCalibrationErrorTerms(M1,A1,M2,A2,M3,A3,
					     					&e00, &e11, &Deltae);
while(1)	
{
//	    printf("attach unknown\n");
	    getchar();
	    VNA_GetReading( freq, &unknown );
	    VNA_ComputeActualReflectionCoefficient( unknown,e00,e11,Deltae,&Rho);
	    VNA_ComputeComplexImpedance(Rho, 50.0+0.0*I, &Zx);
	    VNA_ComputeSWR(Rho, &SWR);

//	    printf("Frequency(MHz) = %f\n",(double)freq/1E6);
//	    printf("Impedance =%f %fi\n",creal(Zx),cimag(Zx));
//	    printf("Swr       =%f \n",SWR);
}
#endif

//done:
    VNA_Close();
	return 0;    
}
#endif
