/*
 *	LCD interface example
 *	Uses routines from delay.c
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 bit mode, with
 *	the hardware connected as follows (the standard 14 pin
 *	LCD connector is used):
 *
 *	PORTB bits 0-3 are connected to the LCD data bits 4-7 (high nibble)
 *	PORTA bit 4 is connected to the LCD RS input (register select)
 *	PORTA bit 5 is connected to the LCD EN bit (enable)
 *
 *	To use these routines, set up the port I/O (TRISA, TRISB) then
 *	call lcd_init(), then other routines as required.
 *
 */
#include <stdio.h>
#include "vna.h"

extern volatile unsigned short int timer_lcd;

// I/O for LCD
#define LCD_RS		_RE5	/* register select */
#define LCD_EN		_RE8	/* enable */
#define LCD_D0		_RE1
#define LCD_D1		_RE2
#define LCD_D2		_RE3
#define LCD_D3		_RE4

/* a very small delay */
void lcd_dly(void)
{
	// 100-200 us delay
	timer_lcd = 2;		
	while(timer_lcd);
}

/* strobe the enable pin */
void lcd_strobe(void)
{
	LCD_EN = 1;
	lcd_dly();
	LCD_EN = 0;
}

/* write a byte to the LCD in 4 bit mode */
void  lcd_write(unsigned char c)
{
	int temp,temp1;
	// send high order nibble out
	temp = PORTE & 0xffe1;
	temp1 = (int)(c >>3) & 0x001e;
	PORTE = temp | temp1;
	lcd_strobe();

	// send low order nibble out
	temp = PORTE & 0xffe1;
	temp1 = (int)(c << 1) & 0x001e;
	PORTE = temp | temp1;
	lcd_strobe();

	// 100-200 us delay
	timer_lcd = 2;		
	while(timer_lcd);
}


/*
 * 	Clear and home the LCD
 */
void lcd_clear(void)
{
	LCD_RS = 0;
	lcd_write(0x1);
	// 20000 us delay
	timer_lcd = 200;		
	while(timer_lcd);
}


/* write a single char to the LCD */
void lcd_putc(const char c)
{
	LCD_RS = 1;		// write chars
	lcd_write(c);
}

/* write a string of chars to the LCD */
void lcd_puts(const char * s)
{
	LCD_RS = 1;		// write chars
	while(*s)
		lcd_write(*s++);
}

/*
 * Go to specified cusor position
 */
void lcd_goto(unsigned char pos)
{
	LCD_RS = 0;
	lcd_dly();
	lcd_write(0x80+pos);
}

/* an i/o test routine for lcd interface */
void lcd_test(void)
{
	int i;

	printf("pulsing RS - pin 4\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_RS = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_RS = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}
	printf("pulsing EN - pin 6\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_EN = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_EN = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}
	printf("pulsing D4 - pin 11\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_D0 = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_D0 = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}
	printf("pulsing D5 - pin 12\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_D1 = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_D1 = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}

	printf("pulsing D6 - pin 13\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_D2 = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_D2 = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}

	printf("pulsing D7 - pin 14\r\n");
	for ( i=0; i < 20; i++)
	{
		LCD_D3 = 0;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
		LCD_D3 = 1;
		// 200ms delay
		timer_lcd = 2000;
		while (timer_lcd);
	}

	printf("ground   - pin 1 should be low\r\n");
	printf("+5 pin   - 2 should be tied high\r\n");
	printf("contrast - pin 3 should about 1 to 2v\r\n");
	printf("r/w      - pin 5 should be tied low\r\n");
}


/* initialise the LCD - put into 4 bit mode */
void setup_lcd(void)
{
	int temp,temp1;

	LCD_EN = 0; // init enable state
	LCD_RS = 0;	// write control bytes

	// 200ms poweron delay
	timer_lcd = 2000;
	while (timer_lcd);

	temp = PORTE & 0xffe1;
	temp1 = (0x0030 >>3) & 0x001e; //0x30 - set 8 bit mode
	PORTE = temp | temp1;
	lcd_strobe();		

	// delay 5ms
	timer_lcd = 50;
	while (timer_lcd);
	lcd_strobe();		//0x30 - set 8 bit mode

	// delay 5ms
	timer_lcd = 50;
	while (timer_lcd);
	lcd_strobe();		//0x30 - set 8bit mode

	// delay 5ms
	timer_lcd = 50;
	while (timer_lcd);

	// set 4 bit mode
	temp = PORTE & 0xffe1;
	temp1 = (0x0020 >>3) & 0x001e; //0x20 - set 4 bit mode
	PORTE = temp | temp1;
	lcd_strobe();		
	lcd_dly();

	lcd_write(0x28);	// 4 bit mode, 2 line mode, 5x7 font
	lcd_write(0x08);	// display off
	lcd_write(0x0e);	// display on,  ul on, blink off
	lcd_write(0x06);	// entry mode = incr, no shift
	lcd_write(0x02);	// display and home cursor
	// delay 5ms
	timer_lcd = 50;
	while (timer_lcd);

}
