//---------------------------------------------------------------------
//	File:		main.c
//
//	Written By:	Lawrence Glaister VE7IT
//
// Purpose: This program is used to control an N2PK vector network analyzer.
//          It is designed to run on a dspic30f4011 40pin pic chip
//      
// 
// The following files should be included in the MPLAB project:
//
//		main.c		    -- Main source code file
//      timer1.c        -- timer 1 setup for 1ms intervals
//      serial.c        -- interface to pc serial port
//      encoder.c       -- interface to quadature encoder
//		pwm.c			-- pwn ch for lcd contrast control
//		VNA-lpg.c		-- actual calcs for VNA interface
//		lcd.c			-- interface routines for LCD display
//		p30f4011.gld	-- Linker script file
//---------------------------------------------------------------------
//
// Revision History
//
// March 11 -- first version 
// note: dont make stack too small... it seems to need close to 500 bytes
//---------------------------------------------------------------------- 
#include <stdio.h>
#include "vna.h"

extern void lcd_goto(int);
extern void int_main( void );

//--------------------------Device Configuration------------------------       
_FOSC(CSW_FSCM_OFF & FRC_PLL8 );  // 8mhz *8 / 4 = 16mips on internal rc osc
_FWDT(WDT_OFF);                   // wdt off
/* enablebrownout @4.2v, 64ms powerup delay, MCLR pin active */
/* pwm pins in use, both active low to give 64ms charge time on bootstrap caps */
_FBORPOR(PBOR_ON & BORV_42 & PWRT_64 & MCLR_EN & RST_PWMPIN & PWMxH_ACT_LO & PWMxL_ACT_LO );

//----------------------------------------------------------------------
extern void setup_TMR1(void);
extern void setup_encoder(void);
extern void setup_uart(void);
extern void setup_lcd(void);
extern void lcd_puts( char *);
extern void lcd_clear( void );


extern volatile unsigned short int timer_test;

extern void setup_pwm(void);
extern void set_pwm(unsigned int amplitude);

extern void setup_adc10(void);

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

void __attribute__((__interrupt__)) _StackError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 0;			
		STATUS_LED = 0;			
		STATUS_LED = 0;			
	}
}

void __attribute__((__interrupt__)) _AddressError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 1;			
		STATUS_LED = 0;			
		STATUS_LED = 0;			
	}
}

void __attribute__((__interrupt__)) _MathError (void)
{
	while (1)
	{
		STATUS_LED = 1;
		STATUS_LED = 0;			
		STATUS_LED = 1;			
		STATUS_LED = 0;			
	}
}

/*********************************************************************
  Function:        void set-io(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        Sets up io ports for in and out

  Note:            Some periperal configs may change them again
********************************************************************/
void setup_io(void)
{
	// PORT B
    ADPCFG = 0xffff;        // make sure analog doesnt grab encoder pins (all digital)
    ADPCFGbits.PCFG0 = 0;   // AN0 is analog in - battery volts
							// 0=output, 1=input
	_TRISB0 = 1;			// used as AN0 above
	_TRISB1 = 0;			// local osc dds to n2pk board
	_TRISB2 = 0;			// rf dds to n2pk board
	_TRISB3 = 0;			// dds reset to n2pk board
	_TRISB4 = 1;			// used by quad encoder input ch A
	_TRISB5 = 1;			// used by quad encoder input ch B
	_TRISB6 = 0;			// wclk to n2pk board
	_TRISB7 = 0;			// dds fqud to n2pk board
	_TRISB8 = 0;			// adcncs to n2pk board

	// PORT C				// 0=output, 1=input
	_TRISC13 = 0;			// serial port aux  tx data
	_TRISC14 = 1;			// serial port aux  rx data
	_TRISC15 = 1;			// spare (maybe future xtal)

	// PORT D				// 0=output, 1=input
	_TRISD0 = 1;			// mode push switch
	_TRISD1 = 1;			// Previous switch
	_TRISD2 = 1;			// Next Switch
	_TRISD3 = 0;			// low = turn on quad encoder led source

	// PORT E				// 0=output, 1=input
	_TRISE0 = 0;			// used by pwm1L to control lcd contrast
	_TRISE1 = 0;			// lcd D0
	_TRISE2 = 0;			// lcd D1
	_TRISE3 = 0;			// lcd D2
	_TRISE4 = 0;			// lcd D3
	_TRISE5 = 0;			// lcd RS
	_TRISE8 = 0;			// lcd EN

	// PORT F				// 0=output, 1=input
	_TRISF0 = 0;			// adc clock to n2pk board
	_TRISF1 = 0;			// onboard test (heartbeat)led
	_TRISF2 = 1;			// PGC used by icsp
	_TRISF3 = 1;			// PGD used by icsp
	_TRISF4 = 1;			// spare
	_TRISF5 = 1;			// spare
	_TRISF6 = 1;			// adc shift data out
}

/*********************************************************************
  Function:        int main(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        main function of the application. Peripherals are 
                   initialized.

  Note:            None.
********************************************************************/
int main(void) 
{
    // set up status LED
	setup_io();			// make all i/o pins go the right dir
	STATUS_LED = 0;		// led on
	setup_uart();		// setup the serial interface to the PC

    setup_TMR1();       // set up 1ms timer
	IEC0bits.T1IE = 1;  // Enable interrupts for timer 1
						// needed for delays in following routines
	// 1/2 seconds startup delay
	timer_test = 5000;		
	while ( timer_test );

	printf("Powerup..timer...");	
	setup_pwm();		// start analog output
	printf("pwm...");
    setup_encoder();    // 16 bit quadrature encoder module setup
	printf("encoder...");
	ENC_LED = 0;		// turn on encoder led
	printf("adc...");
	setup_adc10();		// setup AN0 to read battery volts
	printf("lcd...");
	setup_lcd();		// set up the lcd display
	lcd_clear();
	set_pwm(0);			// LCD contrast (approx 0.5v)
	lcd_puts("Powerup:    VNA-dsp");
	lcd_goto(0x40);
	lcd_puts(VERSION);
	printf("done\r\n");

	// 2 second delay  so user can read lcd msg's
	timer_test = 20000;
	while(timer_test );

	// some junk for the serial channel
	printf("%s%s\n\r",CPWRT,VERSION);
	int_main();		// main menu
	return 0;
}
