//---------------------------------------------------------------------
//	File:		pwm.c
//
//	Written By:	Lawrence Glaister
//
// Purpose: This set of routines to run pwn output
//      
// 
//---------------------------------------------------------------------
//
// Revision History
//
// Nov 21 2005 --    first version Lawrence Glaister
// Mar 18 2006 --    stripped to single output for lcd contrast
//---------------------------------------------------------------------- 
#include "vna.h"
#include <pwm.h>

/* define required pwm rate... dont make it too high as we loose resolution */
#define FPWM 16000

/*********************************************************************
  Function:        void __attribute__((__interrupt__)) _PWMInterrupt(void)

  PreCondition:    None.
 
  Input:           None

  Output:          None.

  Side Effects:    None.

  Overview:        handles pwm interrupts 

  Note:            None.
********************************************************************/
void __attribute__((__interrupt__)) _PWMInterrupt(void)
{
    IFS2bits.PWMIF =0;
}


/*********************************************************************
  Function:        void setupPWM(void)

  PreCondition:    None.
 
  Input:           None

  Output:          None.

  Side Effects:    None.

  Overview:        port setup with deadtime, 
                   fault input disables pwm outputs on a per cycle basis

  Note:            None.
********************************************************************/
void setup_pwm(void)
{
    /* Holds the PWM interrupt configuration value*/
    unsigned int config;
    /* Configure pwm interrupt enable/disable and set interrupt priorties */
    config = (PWM_INT_DIS & PWM_FLTA_DIS_INT & PWM_INT_PR1 & PWM_FLTA_INT_PR0);

    /* clear the Interrupt flags */
    IFS2bits.PWMIF = 0;	
    IFS2bits.FLTAIF = 0;	

    /* Set priority for the period match */
    IPC9bits.PWMIP      = (0x0007 & config);

    /* Set priority for the Fault A */
    IPC10bits.FLTAIP    = (0x0070 & config)>> 4;

    /* enable /disable of interrupt Period match */
    IEC2bits.PWMIE      = (0x0008 & config) >> 3;

    /* enable /disable of interrupt Fault A.*/
    IEC2bits.FLTAIE     = (0x0080 & config) >> 7;


    /* Configure PWM to generate square wave of 50% duty cycle */
    PWMCON2bits.UDIS = 0;
    PDC1 = (FCY/FPWM/2 - 1) ;

    PTPER = (FCY/FPWM/2 - 1);      // set the pwm period register(/2 for cnt up/dwn)
    SEVTCMP = 0x00;
    /* 1 output is independant and enabled */
    PWMCON1 = (PWM_MOD1_IND & PWM_MOD2_IND & PWM_MOD3_IND &  /* independant i/o */
				PWM_PEN1L  & PWM_PDIS1H &                    /* use 1L as pwm, */
				PWM_PDIS2L & PWM_PDIS2H &					 /* rest as I/O */
				PWM_PDIS3L & PWM_PDIS3H 
				);
    /* set dead time options, scale = 1, 10*FCY (about 625ns) */
    DTCON1 = PWM_DTAPS1 & PWM_DTA10;
  
    /* set up the fault mode override bits and mode */
    FLTACON = PWM_FLTA_MODE_CYCLE &
              PWM_FLTA1_DIS &
              PWM_FLTA2_DIS &
              PWM_FLTA3_DIS &
              PWM_OVA1L_INACTIVE &
              PWM_OVA1H_INACTIVE &
              PWM_OVA2L_INACTIVE &
              PWM_OVA2H_INACTIVE &
              PWM_OVA3L_INACTIVE &
              PWM_OVA3H_INACTIVE ;

    /* set special event post scaler, output override sync select and pwm update enable */
    PWMCON2 = (PWM_SEVOPS1 & PWM_OSYNC_PWM & PWM_UEN);
    PTCON   = (PWM_EN & PWM_IDLE_CON & PWM_OP_SCALE1 & PWM_IPCLK_SCALE1 & PWM_MOD_UPDN);

}



/*********************************************************************
  Function:        void set_pwm(unsigned int amplitude)

  PreCondition:    None.
 
  Input:           amplitude value required for pwm

  Output:          None.

  Side Effects:    None.

  Overview:        This routine accepts an 
                   an amplitude value to +1023. From this value,
                   the pwm channel is set as required.

  Note:            None.
********************************************************************/
void set_pwm(unsigned int amplitude)
{
    const long pwm_max = (FCY/FPWM) - 1;      // 100% pwm count
    long temp;

	// check to see if some doorknob passed in a -ve number
	// -ve numbers treated as low output
	if (amplitude > 32767) amplitude = 1;

    /* compute value for pwm register */
    temp = pwm_max - ((pwm_max * (long)amplitude) >> 10);

    /* limit inputs to expected ranges */ 
    if ( temp > pwm_max ) temp = pwm_max;
    if ( temp < 1 ) temp = 1;

//	printf("pwm cmd= %d maxcnt= %ld, writing %ld\r\n",
//			amplitude,pwm_max,temp);
    PDC1 = temp;
}

